/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"
#include "nsISimpleEnumerator.idl"
#include "MailNewsTypes2.idl"

interface nsIMsgDBHdr;

interface nsIDBChangeAnnouncer;

[scriptable, uuid(df64af90-e2da-11d2-8d6c-00805f8a6617)]
interface nsIMsgThread : nsISupports {
  attribute nsMsgKey threadKey;
  attribute unsigned long flags;
  attribute string subject;
  attribute unsigned long newestMsgDate;
  readonly attribute unsigned long numChildren;
  readonly attribute unsigned long numUnreadChildren;

  void AddChild(in nsIMsgDBHdr child, in nsIMsgDBHdr inReplyTo, in boolean threadInThread, in nsIDBChangeAnnouncer announcer);
  nsIMsgDBHdr GetChildAt(in long index);
  nsMsgKey GetChildKeyAt(in long index);
  nsIMsgDBHdr GetChild(in nsMsgKey msgKey);
  nsIMsgDBHdr GetChildHdrAt(in long index);
  nsIMsgDBHdr GetRootHdr(out long index);
  void RemoveChildAt(in long index);
  void RemoveChildHdr(in nsIMsgDBHdr child, in nsIDBChangeAnnouncer announcer);

  void MarkChildRead(in boolean bRead);

  nsIMsgDBHdr GetFirstUnreadChild();

  nsISimpleEnumerator EnumerateMessages(in nsMsgKey parent);

};
